%-------------------------ODESetup.m--------------------------------
%author: XP     create date: 06/16/2005
%modified date:         by:
%---------------------------------------------------------------------
% purpose: Setup ODE equations for PBPK model.
% input    :  t,concentration, PBPK params
% output   :  ODE equations
% units:
%   time   :  minutes; 
%   age    :  year(adult);  or days(infant);
%   weight :  kilogram(bodyWt);  or grams(organ);  or mg(protein)
%   volume :  litre;
%   conc   :  umol/litre;
%   
%--------------------------------------------------------------------- 
%
%  |-----Lung Blood------->-------| 
%  |                              |
%  |                              |
%  |-----Fat---------------<------|
%  |                              |
%  |-----... ------<--------------|
%  |                              |
%  |-----...-------<--------------|
%  |                              |  
%  |-----Brain-----<--------------| 
%  |                              |
%  |-----Kidney----<--------------|
%  |       |                      |
%  |     urine   |-------<--------|
%  |             |                |
%  |--<--Liver---|--intestines--<-|  
%         |                       
%         |                
%         |                  
%     metabolites      
     
function [diff0] = ODESetup(t,conc0,organ,drug,enzyme,protein,const)
%          
%   %------------------------------------------%
%   | initialization and transformation
%   %------------------------------------------%
%
% initialization
assignIndex(); 

numOrgan = organ(1).number;   % the number of organ used 

% conc0 and diff0 are vectors with dimenstion of sizeOfOrgan*sizeOfDrug
% diff0 is a vector to be outputed, containing ODE equations.     
diff0 = [];  

% translate the vector 'conc0' into a matrix 'conc' in a format of conc(iorgan,idrug).
for isolute = 1:drug(1).number
    conc(:,isolute) = conc0((isolute-1)*numOrgan+1 : isolute*numOrgan);    
end
%          
%   %------------------------------------------%
%   | main loop for multiple drugs
%   %------------------------------------------%
%    
for isolute = 1:drug(1).number  % for a specific solute
    diff = zeros(numOrgan,1);   % ODEs for this specific solute
    initialInput = 0;
    
    % use short notation
    for i = 1:numOrgan
        Q(i) = organ(i).Q/1000; 
        V(i) = organ(i).V/1000;        
        parCoeff(i) = organ(i).drug(isolute).parCoeff;
    end    
%      
%   %---------------------------------------------------------------------%
%   | Protein Binding Model                                               |   
%   |   fu = 1/(1+K1*cProt)   if linear binding.                          |
%   |   fu = 1/(1+K1*cProt/(1+K1*fu*conc)); nonlinear binding, solve fu.  |
%   %---------------------------------------------------------------------% 
%
    for iprotein = [ ORM2 ]  %[ORM2 albumin]
        K1 = organ(vein).protein(iprotein).drug(isolute).K1;  %binding constant for the protein( 1/(umoles/liters)); represent the nubmers and affinities of the binding sites to the protein
        cProt = protein(iprotein).ISF*organ(vein).protein(iprotein).conc;  %concentration of protein (ORM2/albumin) at the binding site. (umoles/liters)  
        cDrug = conc(artery,isolute); %total drug concentration in artery

        if K1*cDrug < 0.01
            fu0 = 1/(1+K1*cProt);  %linear binding, since 1+K1*fu*c ~~1
        else  %disp('nonlinear protein binding');                   
            fu0 = 1/2/K1/cDrug*(-1-K1*cProt+K1*cDrug+(1+2*K1*cProt+2*K1*cDrug+K1^2*cProt^2-2*K1^2*cDrug*cProt+cDrug^2*K1^2)^(1/2));            
        end            
        % a safety check.
        %if fu0 < 0 | fu0 > 1.1 disp('error in fu calculation');, end

        for iorgan = 1:organ(1).number   fu(iorgan) = fu0;,   end                
    end  % end of all proteins. 
%          
%   %------------------------%
%   | Metabolic model  
%   %------------------------%
%    
% Michaelis-Menten equation.
%      RAM = ISF*HSF*Vmax*conc/(Km+fu*conc);    
% Vmax is the maximal enzymes rate, 
%      unit: nmol/min/kg body weight in vivo, or;
%      unit: nmol/min/mg of microsomal protein in vitro. 
% Km is the plasma drug concentration at half maximal activity;

    MP = 52.5;     %mg of micosomal protein/g of liver;
    HSF = (MP * organ(liver).V * const.density.liver)^0.74;   %HSF: hepatic scaling factor, mg of microsomal protein, a factor converting intro data to vivo data.  
 
    for i = [ liver ]  %   intestine
        RAM(i,isolute) = 0;        
        for ienzyme = [CYP3A4]    %          
            Vm1 = organ(i).drug(isolute).enzyme(ienzyme).Vm1;
            Km1 = organ(i).drug(isolute).enzyme(ienzyme).Km1;
            concV = fu(i)*conc(i,isolute)/parCoeff(i);    %transform tissue conc to venous conc.        
            RAM(i,isolute) = RAM(i,isolute) + enzyme(ienzyme).ISF*HSF*Vm1*concV/(Km1+concV);
        end
    end 
%          
%   %--------------------------------%
%   | model for drug input, if orally   
%   %--------------------------------%
%                 
    iphase = 1;
    Ka = drug(isolute).Ka;     % 
    oralF = drug(isolute).oralF;        
    dose = drug(isolute).iphase(iphase).dose*const.person.bodyWt/drug(isolute).molWt*1e+3; %uM
    oralInput = Ka*dose*oralF*exp(-Ka*t);     
%  
%   %------------------------------------------%
%   | Derivatives                              |
%   %------------------------------------------%
%    
    % vein 
    % ------------------------ 
    sum0 = 0 ;
    for iorgan = 11:17
        sum0 = sum0 + organ(iorgan).Q./1e+3*conc(artery,isolute); 
    end 
    
    for iorgan = 1:numOrgan
        if Q(iorgan) ~= 0 & parCoeff(iorgan) ~= 0 & ...
                iorgan ~=artery & iorgan ~=vein & iorgan ~=lung
            sum0 = sum0 + Q(iorgan)*conc(iorgan,isolute)/parCoeff(iorgan); 
        end
    end       
    conc(vein,isolute) = sum0/Q(artery);% Mixed venous concentration from all comps.    
    diff(vein,1)= 0;   % venous concentration is actually not used.
    
    % lung 
    % ------------------------    
    i = lung;
    diff(i,1) = Q(artery)*(conc(vein,isolute)-conc(i,isolute)/parCoeff(i));  
    diff(i,1) = diff(i,1)/(V(i));
        
    % blood pool (total)
    % ------------------------ 
    %conc(vein,isolute)
    diff(artery,1) = Q(artery)*(conc(vein,isolute)-conc(artery,isolute));  
    diff(artery,1) = diff(artery,1)/(V(vein)+V(artery));    

    % non-metabolizing organs.
    % ------------------------    
    %selectedOrgans = setdiff(1:numOrgan,[vein  artery lung liver  kidney intestine ]); % 
    for i = 1:numOrgan
        if V(i) ~= 0 & parCoeff(i) ~= 0 & ...
            i ~=artery & i ~=vein & i ~=lung  & ...               
            i ~=liver & i ~=kidney & i ~=intestine              
            diff(i,1) = Q(i)*(conc(artery,isolute)-conc(i,isolute)/parCoeff(i));
            diff(i,1) = diff(i,1)/V(i); % Here conc(i) is the tissue concentration.
        end       
    end        

    % for metabolism in liver
    % ------------------------ 
    i = liver;
    conc_vl = conc(i,isolute)/parCoeff(i);  %conc in venous blood leaving liver
    conc_in = conc(intestine,isolute)/parCoeff(intestine); %conc in venous blood leaving intestine
    Q_hepatic = Q(i)-Q(intestine);  %the hepatic artery blood flow rate.  
    diff(i,1) = Q_hepatic*(conc(artery,isolute)-conc_vl) + Q(intestine)*(conc_in-conc_vl);
    diff(i,1) = (diff(i,1)+oralInput-RAM(i,isolute) )/V(i);                      
        
    % intestine if necessary
    % -----------------------------------------
    i = intestine;    RAM(i,isolute) = 0;    
    diff(i,1) = Q(i)*(conc(artery,isolute)-conc(i,isolute)/parCoeff(i)); 
    diff(i,1) = (diff(i,1) - RAM(i,isolute))/V(i);    
    
    % kidney for excretion 
    % ------------------------   
    i = kidney;
    p1 =      -21.62;  % (-23.34, -19.9)
    p2 =       164.8;  % (152.9, 176.6)
    q1 =      -4.157;  % (-4.217, -4.096) 
    PH = const.person.PH;
    organ(i).drug(isolute).CL  = (p1*PH + p2) / (PH + q1) /1000;  %L/min
    
    %Glomerular Filtration (L/min/kg tissue)      const.GFR_ISF     
    RAM(i,isolute) =   organ(i).drug(isolute).CL * conc(i,isolute)/parCoeff(i);    
    diff(i,1) = Q(i)*(conc(artery,isolute)-conc(i,isolute)/parCoeff(i));
    diff(i,1) = (diff(i,1) - RAM(i,isolute) )/V(i); 
     
    % accumulating ODE equations
    diff0 = [diff0 
             diff];
        
end % end of all drug


end % end of function ODESetup.m.

      